#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <orb/orbit.h>
#include <dirent.h>
#include "tm.h"

/*** App-specific servant structures ***/

typedef struct {
POA_TM_TMPD servant;
PortableServer_POA poa;
} impl_POA_TM_TMPD;

/*** Implementation stub prototypes ***/

static void impl_TM_TMPD__destroy(impl_POA_TM_TMPD *servant,
CORBA_Environment *ev);
static CORBA_char *
impl_TM_TMPD_get_new_protocol(impl_POA_TM_TMPD *servant,
CORBA_char * protocol,
CORBA_Environment *ev);

static TM_protocols*
impl_TM_TMPD_get_protocols(impl_POA_TM_TMPD *servant,
CORBA_Environment *ev);

/*** epv structures ***/

static PortableServer_ServantBase__epv impl_TM_TMPD_base_epv = {
NULL, /* _private data */
NULL, /* finalize routine */
NULL, /* default_POA routine */
};
static POA_TM_TMPD__epv impl_TM_TMPD_epv = {
NULL, /* _private */
(gpointer)&impl_TM_TMPD_get_new_protocol,

(gpointer)&impl_TM_TMPD_get_protocols,

};


/*** vepv structures ***/

static POA_TM_TMPD__vepv impl_TM_TMPD_vepv = {
&impl_TM_TMPD_base_epv,
&impl_TM_TMPD_epv,
};

/*** Stub implementations ***/

static TM_TMPD impl_TM_TMPD__create(PortableServer_POA poa, CORBA_Environment *ev)
{
TM_TMPD retval;
impl_POA_TM_TMPD *newservant;
PortableServer_ObjectId *objid;

newservant = g_new0(impl_POA_TM_TMPD, 1);
newservant->servant.vepv = &impl_TM_TMPD_vepv;
newservant->poa = poa;
POA_TM_TMPD__init((PortableServer_Servant)newservant, ev);
objid = PortableServer_POA_activate_object(poa, newservant, ev);
CORBA_free(objid);
retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

return retval;
}

static void
impl_TM_TMPD__destroy(impl_POA_TM_TMPD *servant, CORBA_Environment *ev)
{
PortableServer_ObjectId *objid;

objid = PortableServer_POA_servant_to_id(servant->poa, servant, ev);
PortableServer_POA_deactivate_object(servant->poa, objid, ev);
CORBA_free(objid);

POA_TM_TMPD__fini((PortableServer_Servant)servant, ev);
g_free(servant);
}

static CORBA_char *
impl_TM_TMPD_get_new_protocol(impl_POA_TM_TMPD *servant,
CORBA_char * protocol,
CORBA_Environment *ev)
{
  CORBA_char * retval;
  g_print("Get_New_Protocol called. Someone wants a/an %s object.\n", protocol);
  g_print("Retval: %s\n", retval);
  return retval;
}


static TM_protocols*
impl_TM_TMPD_get_protocols(impl_POA_TM_TMPD *servant,
CORBA_Environment *ev){
  TM_protocols* retval;
//  DIR *dir;
//  dirent *ent;
  g_print("Get_Protocols called.\n");
//  dir=opendir("/etc/tm/protocols/");
//  while ((ent = readdir(dir)) != NULL){
//    g_print("%s\n",ent->d_name);
//  }
//  closedir(dir);
  return retval;
}


/* Main */

int main (int argc, char *argv[]){ 
  CORBA_ORB orb;
  TM_TMPD myTMPD = NULL;
  CORBA_Environment ev;
  PortableServer_POA poa;
  FILE * ofp;
  char *stringified_reference;
  char buffer[1024];
  signal(SIGINT, exit);
  signal(SIGTERM, exit);
  CORBA_exception_init(&ev);
  orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
  poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
  PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(poa, &ev), &ev); 
  myTMPD = impl_TM_TMPD__create(poa, &ev);
  if (!myTMPD) {
    printf("Cannot get objref\n");
    return 1;
  }
  stringified_reference = CORBA_ORB_object_to_string(orb, myTMPD, &ev);
  strcpy(buffer,g_get_home_dir());
  strcat(buffer, "/.tm/tmpd.ior");
  ofp = fopen(buffer,"w");
  if (ofp == NULL){
    g_print("Could not open ~/.tm/tmpd.ior\nDoes ~/.tm/ exist?\n");
    return 1;
  }
  fprintf(ofp,"%s", stringified_reference);
  fclose(ofp);
  g_print("Written the file ~/.tm/tmpd.ior with the IOR of this server.\nNow waiting for requests...\n"); fflush(stdout);
  CORBA_free(stringified_reference);
  CORBA_ORB_run(orb, &ev);
  return 0;
}
